// $Id: popup_qa_grade_ajx.js,v 1.1 2015/09/29 13:11:45 marcos Exp $ 
// 
// Here are the pure javascript functions (i.e. does not contain anything
// about context, label decoder and so on.

function initPage() {

    // Initialize the recording tag list
    initTagEditor(udpdateAudioRecordingsTagsLine);

    // Initialize comments
    initCommentsStatus();
    initCommentsOnQuestionsTitles();

    // Initialize section sums and averages
    foldSectionSums();

    // Initialize the event handlers
    initEventHandlers();

    // Init the local items visibility array
    initLocalItemsVisibility();

    // Highlight shortcuts
    highlightShortcuts();

    // Highlight zero weight questions
    highlightZeroWeightQuestions();

    // Initializa auxiliary information
    initAuxiliaryInfo();

    // Update dynamic values
    if (formInEditMode === 1) {
        recount(); 
    } else {
        updatePage();            
    }

    // Update comments
    updateComments();

    // Keep the section alive
    keepSessionAlive();
}

function updatePage() {

    // Update ratings
    updateRatings();

    // Update items visibility
    updateFormItemsVisibility();

    // Update items statuses
    updateItemsStatus();

    // Update sections sum and average values
    updateSectionsValues();

    // Update the total score and total form average
    updateTotalSumAndAvg();

    // Update Save button status
    updateSaveButtonStatus();

    // Update tag recordings page
    udpdateAudioRecordingsTagsLine();
}

function initCommentsStatus() {

    var commentsEnabled = document.getElementById("QAE_commentsEnabled").value;

    if (commentsEnabled == "1") {
        enablecomments("1");
    } else {
        enablecomments("0");
    }
}


function initEventHandlers() {

    $("#QAE_formId").bind("change", OnChangeCboForm);
    $("select[id^=QAE_item]").bind("change", recount);
    $("input[id^=QAE_item]:not(:radio):not(:checkbox):not(:hidden)").bind("change", recount);

    if (formInEditMode === 1) {
        $("input[id^=checkbox_rt_check]").bind("click", hideShowItemsByLevel);
    }
}

function hideShowItemsByLevel() {

    var name = this.id;
    var value = this.checked;
    var subName = name.substring(0, name.length - 2);
    var sectionNum = name.substring(name.length - 1, name.length)-1;

    var boundary = json_sources['rt_boundaries'][sectionNum];
    var minVal = boundary[0];
    var maxVal = boundary[1];

    updateLocalItemsVisibility(minVal, maxVal, value);
    updateFormItemsVisibility();
    updateGroupsVisibilityUserPreferences();
}

// To enable the show/hide option for items based on ratings
// we use a local generated array that overrides the visibility for each
// item in the form.
var localItemsVisibility = [];
function initLocalItemsVisibility() {

    for (var i = 0; i < json_sources["itemsPosition"].length; i++) {
        localItemsVisibility[i] = true;
    }
}

function updateLocalItemsVisibility(minVal, maxVal, hide) {

    if (formInEditMode === 1) {
        for (var i = 0; i < localItemsVisibility.length; i++) {
            if ((json_sources['itemsVals'][i] <= maxVal) && (json_sources['itemsVals'][i] >= minVal)) {
                localItemsVisibility[i] = hide;
            }
        }
    }
}

function updateRatings() {

    // Update the ratings cells
    var result = $.map($(".ST_TR0 > td > div[id^=rt_]"), function(value) {
        var name = value.id;
        var subName = name.substring(0, name.length - 2);
        var sectionNum = name.substring(name.length - 1, name.length)-1;

        if (subName == "rt_rating") {
            value.innerHTML = "<font color='" + json_sources[subName + "_colors"][sectionNum] + "'>" + json_sources[subName][sectionNum] + "</font>";
        } else {
            value.innerHTML = json_sources[subName][sectionNum];
        }
        return value;
    });

    // Hide ratings rows with empty label (they are not supposed to be shown if empty)
    result = $.map($("[id^=rt_row_rating_]"), function(value) {
        var name = value.id;
        var sectionNum = name.substring(name.length - 1, name.length)-1;
        var label = json_sources['rt_rating'][sectionNum];
        var style = "";
        if (label.length == 0) {
            style = "none";
        }
        $(value).css("display", style);
    });

    // Update the ratings checkboxes
    result = $.map($("input[id^=checkbox_rt_check]"), function(value) {
        var name = value.id;
        var sectionNum = name.substring(name.length - 1, name.length)-1;
        value.checked = json_sources["rt_visibility"][sectionNum];

        var boundary = json_sources['rt_boundaries'][sectionNum];
        var minVal = boundary[0];
        var maxVal = boundary[1];

        updateLocalItemsVisibility(minVal, maxVal, value.checked);

        return value;
    });
}

function updateFormItemsVisibility() {

    for (var i = 0; i < json_sources["itemsPosition"].length; i++) {
        var itemPosition = json_sources["itemsPosition"][i];
        var itemCode = json_sources["itemsCode"][itemPosition];

        var style = ((json_sources["itemsVisibility"][itemPosition] != 0) &&
                       ((json_sources["itemsMandatory"][itemPosition] == 0) || !hideNAQuestions) )? "" : "none";

        // Local visibility overrides global items visibility (except for N/A values that are not classified by ratings)
        if (json_sources["itemsMandatory"][itemPosition] == 0) {
            style = (localItemsVisibility[itemPosition] == true)? style : "none";
        }

        $("tr:has(td[id=QAE_div_code_" + itemCode + "])").css("display", style);
    }

    $("tr:has(td[id*=QAE_div_code]):not(:hidden)").removeClass("ST_TR0 ST_TR1");
    $("tr:has(td[id*=QAE_div_code]):not(:hidden):even").addClass("ST_TR1");
    $("tr:has(td[id*=QAE_div_code]):not(:hidden):odd").addClass("ST_TR0");
}

function updateItemsStatus() {

    for (var i = 0; i < json_sources["itemsPosition"].length; i++) {
        $("div[id=QAE_div_" + json_sources["itemsCode"][i] + "]:first").html(json_sources["itemsStatus"][i]).css("color", json_sources["itemsStatusColor"][i]).css("text-align", "center");
        $("input[id=QAE_item_" + json_sources["itemsCode"][i] + "]:first").css("background", json_sources["itemsBackgroundColor"][i]);
        $("select[id=QAE_item_" + json_sources["itemsCode"][i] + "]:first").css("background", json_sources["itemsBackgroundColor"][i]);
        $("div[id=QAE_wrap_" + json_sources["itemsCode"][i] + "]:first").css("background", json_sources["itemsBackgroundColor"][i]);
    }
}

function foldSectionSums() {

    var result = $.map($(".ST_TR0[id^=par_avg_row]"), function(value) {
        var name = value.id;
        var sectionNum = name.substring("par_xxx_row".length, name.length)-1;
        var sectionLen = json_sources["sectionsLengths"][sectionNum];
        if (sectionLen == 0)
            value.style.display = "none";
        return value;
    });
}

function highlightShortcuts() {

    var result = $.map($(".stData[id^=QAE_div_code], .stData[id^=QAE_div_text]"), function(value) {
        var name = value.id;
        var itemName = name.substring("QAE_div_xxxx_".length, name.length);
        var itemPos = 0;
        for (var i = 0; i < json_sources["itemsPosition"].length; i++) {
            if (json_sources["itemsCode"][i] == itemName) {
                itemPos = json_sources["itemsPosition"][i];
                break;
            }
        }
        var shortcut = json_sources["shortcuts"][itemPos];
        if (shortcut != 0)
            value.style.fontStyle = "italic";

        return value;
    });
}

function highlightZeroWeightQuestions() {

    var result = $.map($(".stData[id^=QAE_div_code], .stData[id^=QAE_div_text]"), function(value) {
        var name = value.id;
        var itemName = name.substring("QAE_div_xxxx_".length, name.length);
        var itemPos = 0;
        for (var i = 0; i < json_sources["itemsPosition"].length; i++) {
            if (json_sources["itemsCode"][i] == itemName) {
                itemPos = json_sources["itemsPosition"][i];
                break;
            }
        }
        var weight = json_sources["itemsWeight"][itemPos];
        if (weight == 0)
            $(value).css("color", "gray");

        return value;
    });
}

function initAuxiliaryInfo() {

    var style = (json_sources['deleted'] != 0)? "block" : "none";
    $("#formDeleted").css("display", style);

    style = (alreadyFilled)? "block" : "none";
    $("#formAlreadyFilled").css("display", style);
}

function updateSectionsValues() {

    var result = $.map($(".ST_TR0 > td > [id^=QAE_par_sum]"), function(value) {
        var name = value.id;
        var sectionNum = name.substring("QAE_par_sum".length, name.length)-1;
        var sectionSum = json_sources["sectionsSums"][sectionNum];
        var sectionAvg = json_sources["sectionsAvgs"][sectionNum];
        if ((sectionSum == 0) && (sectionAvg == 0)) {
            value.innerHTML = "<font color='red'>" + sectionSum + "</font>";
        } else {
            value.innerHTML = sectionSum;
        }
        return value;
    });

    result = $.map($(".ST_TR0 > td > [id^=QAE_par_avg]"), function(value) {
        var name = value.id;
        var sectionNum = name.substring("QAE_par_avg".length, name.length)-1;
        var sectionAvg = json_sources["sectionsAvgs"][sectionNum];
        var sectionSum = json_sources["sectionsSums"][sectionNum];
        if ((sectionSum == 0) && (sectionAvg == 0)) {
            value.innerHTML = "<font color='red'>" + sectionAvg + "</font>";
        } else {
            value.innerHTML = sectionAvg;
        }
        return value;
    });
}

function updateTotalSumAndAvg() {

    $(".ST_TR0 > td > span[id=tot_sum]").html(json_sources["tot_sum"] + "/" + json_sources["tot_maxsum"]);
    $(".ST_TR0 > td > span[id=tot_avg]").html(json_sources["tot_avg"]);
}

function updateSaveButtonStatus() {

    $("#btnSave").each(function() {
        if (typeof this.disabled != "undefined")
                this.disabled = (json_sources["completed"] == 0)? true : false;
    });
}

function getRadioValue( stName ) {

    stName = "QAE_item_" + stName;
    stRadioName = stName + "_radio";

    var value = $("input[id^=" + stRadioName + "]:checked").val();
    $("#" + stName).val(value);
}

function disableUITransaction( disable ) {

}

function togglecomments() {

    if (document.getElementById("qaform").style.display == "block") {
        enablecomments("1");
    } else {
        enablecomments("0");
    }
}

function addjob(astCallId) {

    var jobId = $("#QAE_jobId").val();

    var params = {
        TPF_VERBO: "qm_3p_addcall",
        TPF_XPAR1: 's',
        CODA_add_to_export_job: jobId,
        JOB_CALLID: astCallId
    }

    $.ajax( {
        type:        "post",
        url:         "../tpf",
        contentType: "application/x-www-form-urlencoded; charset=UTF-8",
        dataType:    "json",
        data:        params,
        success: function(resp) {

            var result = resp['jobresult'];
            if (result.length != 0)
                alert(result);

            disableUITransaction(false);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {

            disableUITransaction(false);
        }
    });
}

function startprint() {

    document.getElementById("qaform").style.display = "block";
    document.getElementById("comments").style.display = "block";

    print();
}

function recount() {

    disableUITransaction(true);

    var parameters = collectInfoForPOST("qm_qa_popup_ds");

    $.ajax( {
        type:        "post",
        url:         "../tpf",
        contentType: "application/x-www-form-urlencoded; charset=UTF-8",
        dataType:    "json",
        data:        parameters,
        success: function(resp) {

            json_sources = resp;
            updatePage();

            disableUITransaction(false);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {

            disableUITransaction(false);
        }
    });
}

function updateGroupsVisibilityUserPreferences() {

    // Retrieve all visibility checkbox statuses
    var stringArray = [ 'Y', 'Y', 'Y', 'Y' ];
    var items = $("input[id^=checkbox_rt_check]");
    $.map(items, function(value) {

        var name = value.id;
        var sectionNum = name.substring(name.length - 1, name.length)-1;
        var checked = value.checked;

        stringArray[sectionNum] = ((checked)? 'Y' : 'N');
    });

    // and save them at server side for future use
    var parameters = {
        TPF_VERBO: "qm_qa_popup_update_visibility_userprefs",
        TPF_XPAR1: '',
        QAE_VisibleGroups: stringArray.join("")
    }
    $.ajax( {
        type:        "post",
        url:         "../tpf",
        contentType: "application/x-www-form-urlencoded; charset=UTF-8",
        dataType:    "json",
        data:        parameters,
        success: function(resp) {

            // Nothing to do with the answer.
            // (the form already reconfigure itself)
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {

        }
    });
}

function runSave() {

    $("#QAE_tot_avg").val(json_sources['tot_avg']);
    $("#qa_dtform").trigger("submit");
}

function runCompact() {

    hideNAQuestions = !hideNAQuestions;
    updateFormItemsVisibility();
}

function collectInfoForPOST(verb) {

    // Generate the parameter string
    var items = $("input[id^=QAE_item]:not(:radio):not(:checkbox), select[id^=QAE_item]");

    var params = {
        TPF_VERBO: verb,
        TPF_XPAR1: '',
        QAE_mode: "I"
    }

    $.map(items, function(value) {
        params[value.id] = value.value;
    });

    $.map($("input[id^=QAE_item_mdty]"), function(value) {
       params[value.id] = value.checked;
    });

    return params;
}

function OnChangeCboForm() {

    var commentsEnabled = document.getElementById("QAE_commentsEnabled").value;
    var formId = document.getElementById("QAE_formId").value;
    document.location.href = "qm_qa_popup.do?QAE_mode=I" + "&QAE_formId=" + formId + "&QAE_commentsEnabled=" + commentsEnabled;
}

function keepSessionAlive() {
    oHiddenImg = document.getElementById( "keepalive" );
    oHiddenImg.src = "qm_keepalive.do?cachebrk=" + Math.random();
    setTimeout("keepSessionAlive()", 120 * 1000);
}

function updateComments() {
    $("#oldcomments").html(json_sources_comments['comments']);
    var style = (json_sources_comments['comments'].length != 0)? "block" : "none";
    $("#divBtnRemove").css("display", style);
}

function insertNewComment() {

    var parameters = {
        TPF_VERBO: "qm_qa_popup_addcomment",
        TPF_XPAR1: '',
        QAE_mode: "C"
    }

    $.map($("#QAE_comment"), function(value) {
        parameters[value.id] = value.value;
    });

    $.ajax( {
        type:        "post",
        url:         "../tpf",
        contentType: "application/x-www-form-urlencoded; charset=UTF-8",
        dataType:    "json",
        data:        parameters,
        success: function(resp) {

            json_sources_comments = resp;
            updateComments();
            $("#QAE_comment").val("");

            disableUITransaction(false);
        },
        error: function (XMLHttpRequest, textStatus, errorThrown) {

            disableUITransaction(false);
        }
    });
}

function closeCommentDialog() {
$("#commentDlg").dialog( "close" );
}

function initCommentsOnQuestionsTitles() {

for (var prop in json_sources_comments) {
    if (json_sources_comments.hasOwnProperty(prop)) {
        var codePos = prop.indexOf("commentstt_");
        if (codePos != -1) {
            var itemCode = prop.substring(11);
            var commentText = json_sources_comments[prop];

            /* Set a div title */
            $("#QAE_div_comments_"+itemCode).attr('title', commentText);

            /* Show\hide the icons */
            $("#QAE_icon_comments_"+itemCode).css("display", "block");
            $("#QAE_icon_addcomments_"+itemCode).css("display", "none");
        }
    }
}
}

function updateCommentsForCurrentItem() {

itemCode = $("#QAE_commentItemId").val();
updateCommentsTableForItem(itemCode);
}

function updateCommentsTableForItem(itemCode) {

// Search the comment in the comment list
if (json_sources_comments.hasOwnProperty('comments_' + itemCode)) {
    commentText = json_sources_comments['comments_' + itemCode];
    $("#QAE_itemCommentHistoricalArea").html(commentText);

} else
    $("#QAE_itemCommentHistoricalArea").html("");
}

function insertNewCommentForCurrentItem() {

var itemCode = $("#QAE_commentItemId").val();
var formId = $("#QAE_formId").val();

var parameters = {
    TPF_VERBO: "qm_qa_popup_addcomment",
    TPF_XPAR1: '',
    QAE_mode: "Q",
    QAE_commentItemId: itemCode,
    QAE_formId: formId,
    QAE_commentsEnabled: 0
}

$.map($("#QAE_itemCommentTextArea"), function(value) {
    parameters[value.id] = value.value;
});

$.ajax( {
    type:        "post",
    url:         "../tpf",
    contentType: "application/x-www-form-urlencoded; charset=UTF-8",
    dataType:    "json",
    data:        parameters,
    success: function(resp) {

        json_sources_comments = resp;
        updateCommentsForCurrentItem();
        initCommentsOnQuestionsTitles();
        updateComments();
        $("#QAE_itemCommentTextArea").val("");

        disableUITransaction(false);
    },
    error: function (XMLHttpRequest, textStatus, errorThrown) {

        disableUITransaction(false);
    }
});
}

function startPlayer( name, url, offset ) {

    startPlayerInner( name, url, offset );    
    // $( "#tabs" ).tabs( "option", "selected", 2 ); 
    enablecomments("1");
}    
